/*
Copyright (c) 2010 Giuseppe Calà <jiveaxe@gmail.com>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/* Project related */
#include "plugin_readitlater.h"
#include "configdialog.h"


/* Kde related */
#include <kgenericfactory.h>
#include <khtml_part.h>
#include <kparts/partmanager.h>
#include <KActionCollection>
#include <KActionMenu>
#include <knotification.h>

/* QT related */
#include <QApplication>

K_PLUGIN_FACTORY( ReadItLaterFactory, registerPlugin< Plugin_ReadItLater >(); )
K_EXPORT_PLUGIN( ReadItLaterFactory( "readitlater" ) )

Plugin_ReadItLater::Plugin_ReadItLater(QObject *parent, const QVariantList & /*args*/) :
Plugin(parent), m_shortenJob(0)
{

	m_part = dynamic_cast<KHTMLPart *>(parent);

	if(!m_part)
	{
		kDebug() << "couldn't get KHTMLPart";
		return;
	}

	KActionMenu *menuTools = new KActionMenu(KIcon("readitlater"), i18n("Read It Later"), actionCollection());
	actionCollection()->addAction("action readitlaterToolsMenu", menuTools);

	KActionMenu *menuToolBar = new KActionMenu(KIcon("readitlater"), i18n("Read It Later"), actionCollection());
	actionCollection()->addAction("action readitlaterToolBar", menuToolBar);

	menuTools->setDelayed( false );
	menuToolBar->setDelayed( false );

	QAction *sendToInstapaperAction = actionCollection()->addAction("send_to_instapaper");
	sendToInstapaperAction->setText(i18n("Send to InstaPaper"));
	connect(sendToInstapaperAction, SIGNAL(triggered()), SLOT(slotSendToInstapaper()));
	menuTools->addAction(sendToInstapaperAction);
	menuToolBar->addAction(sendToInstapaperAction);

	QAction *sendAllToInstapaperAction = actionCollection()->addAction("send_all_to_instapaper");
	sendAllToInstapaperAction->setText(i18n("Send all tabs to InstaPaper"));
	connect(sendAllToInstapaperAction, SIGNAL(triggered()), SLOT(slotSendAllToInstapaper()));
	menuTools->addAction(sendAllToInstapaperAction);
	menuToolBar->addAction(sendAllToInstapaperAction);

	QAction *goToInstapaperAction = actionCollection()->addAction("go_to_instapaper");
	goToInstapaperAction->setText(i18n("Go to InstaPaper"));
	connect(goToInstapaperAction, SIGNAL(triggered()), SLOT(slotGoToInstapaper()));
	menuTools->addAction(goToInstapaperAction);
	menuToolBar->addAction(goToInstapaperAction);

	QAction *configureInstapaperAction = actionCollection()->addAction("configure_instapaper");
	configureInstapaperAction->setText(i18n("Configure InstaPaper"));
	connect(configureInstapaperAction, SIGNAL(triggered()), SLOT(slotConfigureInstapaper()));
	menuTools->addAction(configureInstapaperAction);

	// KConfig file
	m_cfg = new KConfig( "readitlaterrc", KConfig::NoGlobals );
}

Plugin_ReadItLater::~Plugin_ReadItLater()
{
	delete m_shortenJob;
}

void Plugin_ReadItLater::slotSendToInstapaper()
{
	KConfigGroup group = m_cfg->group ("General");
	if (group.readEntry("Username", QString()) == "") {
		// open dialog to insert username e password
		ConfigDialog *dlg = new ConfigDialog(m_part->widget(), m_part);
		connect(dlg, SIGNAL( accountData(const QString&, const QString&) ), this, SLOT( editAccount(const QString&, const QString&) ));
		dlg->exec();
		delete dlg;
	}
	else {
		startJob(m_part->baseURL(), m_part->executeScript("document.title").toString());
	}
}

void Plugin_ReadItLater::slotSendAllToInstapaper()
{
	KConfigGroup group = m_cfg->group ("General");
	if (group.readEntry("Username", QString()) == "") {
		// open dialog to insert username e password
		ConfigDialog *dlg = new ConfigDialog(m_part->widget(),m_part);
		connect(dlg, SIGNAL( accountData(const QString&, const QString&) ), this, SLOT( editAccount(const QString&, const QString&) ));
		dlg->exec();
		delete dlg;
	}
	else {
		KParts::PartManager* pm = m_part->manager();
		const QList<KParts::Part *> parts = pm->parts();
		for(int i = 0; i < parts.size(); i++) {
			KHTMLPart* const part = static_cast<KHTMLPart*>( parts.at(i));
			startJob(part->baseURL(), part->executeScript("document.title").toString());
		}
	}
}

void Plugin_ReadItLater::slotGoToInstapaper()
{
	m_part->executeScript("window.open('http://www.instapaper.com/u')");
}

void Plugin_ReadItLater::slotConfigureInstapaper()
{
	KConfigGroup group = m_cfg->group ("General");
	ConfigDialog *dlg = new ConfigDialog(m_part->widget(),m_part, group.readEntry("Username", QString()), group.readEntry("Password", QString()));
	connect(dlg, SIGNAL( accountData(const QString&, const QString&) ), this, SLOT( editAccount(const QString&, const QString&) ));
	dlg->exec();
	delete dlg;
}

void Plugin_ReadItLater::editAccount(const QString& username, const QString& password)
{
	KConfigGroup group = m_cfg->group ("General");
	group.writeEntry("Password", password);
	group.writeEntry("Username", username);
	m_cfg->sync();
}

void Plugin_ReadItLater::startJob(const KUrl url, const QString title)
{
	KConfigGroup group = m_cfg->group ("General");
	if(!url.url().startsWith("file:") && url.host() != "www.instapaper.com") {
		KUrl ip_url("https://www.instapaper.com/api/add");
		m_username = group.readEntry("Username", QString());
		m_password = group.readEntry("Password", QString());

		m_shortenJob = KIO::storedGet(ip_url.url() + "?"
		+ "username=" + KUrl::toPercentEncoding(m_username)
		+ "&password=" + KUrl::toPercentEncoding(m_password)
		+ "&url=" + KUrl::toPercentEncoding(url.url())
		+ "&title=" + KUrl::toPercentEncoding(title), KIO::NoReload, KIO::HideProgressInfo);
		connect( m_shortenJob, SIGNAL( result( KJob* ) ), SLOT( savePage( KJob* ) ) );
	}
}

void Plugin_ReadItLater::savePage( KJob* job )
{
	if ( !m_shortenJob ) return; //track changed while we were fetching

	// It's the correct job but it errored out
	if ( job->error() != KJob::NoError && job == m_shortenJob ) {
		qDebug() << "Job Error: " << job->errorText();
		return;
	}

	// not the right job, so let's ignore it
	if ( job != m_shortenJob )
		return;

	if ( job ) {
		KIO::StoredTransferJob* const storedJob = static_cast<KIO::StoredTransferJob*>( job );
		int responseCode = storedJob->metaData().value("responsecode").toInt();
		switch(responseCode) {
			case 201:
			{
				qDebug() << "This URL has been successfully added to this Instapaper account.";
				QPixmap pix = KIcon("readitlater").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
				KNotification::event(KNotification::Notification, i18n("<p>Page added to your Instapaper account</p>"), pix, 0);
				break;
			}
			case 400:
			{
				qDebug() << "ERROR: Bad request. Probably missing a required parameter, such as url.";
				QPixmap pix = KIcon("process-stop").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
				KNotification::event(KNotification::Error, i18n("Instapaper bad request.\nProbably missing a required parameter, such as url"), pix,0, KNotification::CloseOnTimeout);
				break;
			}
			case 403:
			{
				qDebug() << "ERROR: Invalid username or password.";
				QPixmap pix = KIcon("process-stop").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
				KNotification::event(KNotification::Error, i18n("Instapaper invalid username or password"), pix,0, KNotification::CloseOnTimeout);
				break;
			}
			case 500:
			{
				qDebug() << "ERROR: The service encountered an error. Please try again later.";
				QPixmap pix = KIcon("process-stop").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium);
				KNotification::event(KNotification::Error, i18n("Instapaper\nThe service encountered an error. Please try again later."), pix,0, KNotification::CloseOnTimeout);
				break;
			}
			default:
				qDebug() << "Unknown responseCode = " << responseCode;
		}
	}
	else {
		qDebug() << "No data";
	}

	m_shortenJob = 0;
}


#include "plugin_readitlater.moc"
